/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     ltree_selected_dataH
#define     ltree_selected_dataH

#include    "main.h"
#include    "push_back.h"

namespace tree{

//-----------------------------------------------------------------------------
// Item of an entry of a data selection - a vector of elements

class TSelectedDataEntryItem : public vector<string>{

    public:

        string Stringify(const string left_border="{",
            const string right_border="}", const string delim=",") const;

        TSelectedDataEntryItem(void){}
        TSelectedDataEntryItem(const size_t s){
            this->resize(s);
            }
    };

//-----------------------------------------------------------------------------
// Entry of a data selection - a vector of items, or a matrix of elements

class TSelectedDataEntry : public vector<TSelectedDataEntryItem>{

    public:

        string Stringify(const string left_border="{",
            const string right_border="}", const string delim=",",
            const bool avoid_empty_lines=true) const;
        void PushAll(const size_t i, const TSelectedDataEntryItem &src);

        void Reset(const size_t s){
            this->clear();
            this->resize(s);
            }

        TSelectedDataEntry(void){}
        TSelectedDataEntry(const size_t s){
            this->resize(s);
            }
    };

//-----------------------------------------------------------------------------
// Selected data - a vector of entries, or a matrix of items

class TSelectedData : public vector<TSelectedDataEntry>{

    public:

        string Stringify(const string left_border="{",
            const string right_border="}", const string delim=",",
            const bool avoid_empty_lines=true) const;
    };

//-----------------------------------------------------------------------------

} // namespace tree

#endif
