/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     ltree_tree_costructorH
#define     ltree_tree_costructorH

#include    "main.h"
#include    "branch.h"
#include    <iostream>
#include    <algorithm>
#include    <map>

#define     TYPE_XML            0
#define     TYPE_JSON           1

namespace tree{

pair<vector<TBranchDraft>, string> ExpandBranchDrafts(
    const vector<TBranchDraft> &drafts);

//-----------------------------------------------------------------------------
// Tree of nodes

class TTreeConstructor{

    private:

        vector<TNode> nodes;      // Nodes of the tree
        vector<TBranch> branches; // Branches of the tree (every branch is a mass of ids of sequented nodes)
        int doc_type;             // Type of document this tree is constructed for

        // Test a specified/given node
        bool IsPresent(TNode &node) const;

        // Find a node of this tree using given info
        TNode* FindNode(const string &node_name, const int parent_id);
        TNode* FindNode(const int id);
        TNode* FindRoot(void){
            return FindNode(0);
            }
        TNode* FindNodeOnLevel(const string &node_name, const int level);

        // Get tree level of the node
        int GetLevel(const TNode &node);

        // Formation of a tree
        TNode* NewNode(const string &name, TNode *parent=NULL);
        TNode* EndNode(const string &name, const int branch_mark,
                const int master_branch=-1, const bool index_enabled=false,
                const bool is_anchor_branch=false, TNode *parent=NULL);
        bool NewAttribute(const string &attr, TNode *node);
        bool NewBranch(TNode *leaf, const int branch_mark,
                const int master_branch=-1, const bool index_enabled=false,
                const bool is_anchor_branch=false);
        bool TraceParents(TNode *node, vector<TNode> &parents);

        #ifdef LTREE_DEBUGGING
        void PrintTree(TNode *node=NULL, const int indent_level=0);
        void PrintBranches(const int indent_level=0) const;
        #endif

    public:

        // Construct a tree using given base nodes and a branch draft
        pair<pair<vector<TNode>, vector<TBranch>>, bool> Construct(
                const string &root_name,
                const vector<TBranchDraft> &branch_drafts,
                const int dt);
    };

//-----------------------------------------------------------------------------

} // namespace tree

#endif
